package bruhcollective.itaysonlab.jetispot.ui.screens.hub

import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.mutableStateOf
import androidx.hilt.navigation.compose.hiltViewModel
import bruhcollective.itaysonlab.jetispot.core.SpPlayerServiceManager
import bruhcollective.itaysonlab.jetispot.core.api.SpInternalApi
import bruhcollective.itaysonlab.jetispot.core.api.SpPartnersApi
import bruhcollective.itaysonlab.jetispot.core.collection.db.LocalCollectionDao
import bruhcollective.itaysonlab.jetispot.core.objs.player.PlayFromContextData
import bruhcollective.itaysonlab.jetispot.core.util.Log
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.cancel
import kotlinx.coroutines.flow.stateIn
import kotlinx.coroutines.launch
import xyz.gianlu.librespot.metadata.AlbumId
import javax.inject.Inject

@Composable
fun AlbumScreen(
  id: String,
  viewModel: AlbumViewModel = hiltViewModel()
) {
  LaunchedEffect(Unit) {
    viewModel.load(id)
  }

  HubScaffold(
    appBarTitle = viewModel.title.value,
    state = viewModel.state,
    viewModel = viewModel
  ) {
    viewModel.reload(id)
  }
}

@HiltViewModel
class AlbumViewModel @Inject constructor(
  private val spInternalApi: SpInternalApi,
  private val spPartnersApi: SpPartnersApi,
  private val spPlayerServiceManager: SpPlayerServiceManager,
  private val spDao: LocalCollectionDao
) : AbsHubViewModel(), CoroutineScope by MainScope() {
  val title = mutableStateOf("")

  private fun subscribeOnAlbum(id: String) {
    launch {
      spDao.subscribeOnAlbum(AlbumId.fromBase62(id).hexId()).stateIn(this).collect {
        Log.d("AlbumViewModel", "state = $it")
        mainAddedState.value = it.isNotEmpty()
      }
    }
  }

  override fun onCleared() {
    cancel()
  }

  suspend fun load(id: String) = load {
    subscribeOnAlbum(id)
    loadInternal(id)
  }

  suspend fun reload(id: String) = reload { loadInternal(id) }
  private suspend fun loadInternal(id: String) = spInternalApi.getAlbumView(id).also { title.value = it.title ?: "" }

  override fun play(data: PlayFromContextData) = play(spPlayerServiceManager, data)
  override suspend fun calculateDominantColor(url: String, dark: Boolean) = calculateDominantColor(spPartnersApi, url, dark)
}